//---------------------------------------------
// FracNEON_sp_opt1 Benchmark by Michael Kübel 
//---------------------------------------------

#include <iostream>
#include <iomanip>
#include <cstdint>
#include <SDL2/SDL.h>
#define WINDOW_WIDTH 600

using namespace std;

extern "C" uint64_t CalcMandelbrotASM_FracNEON_sp_opt1(const uint32_t* p);

int main(int argc, char** argv)
{
	const char prg_info[] = "FracNEON_sp_opt1 V1.0 by Michael Kübel";
    uint32_t *pixels = new uint32_t[WINDOW_WIDTH*WINDOW_WIDTH];    //32 Bit pixel array mapped to texture
    memset(pixels,0,WINDOW_WIDTH*WINDOW_WIDTH*sizeof(uint32_t)); //set all to pixels to 0
	
    SDL_Event event;
    SDL_Init(SDL_INIT_VIDEO);
    SDL_Window   *window   = SDL_CreateWindow(prg_info,SDL_WINDOWPOS_UNDEFINED,SDL_WINDOWPOS_UNDEFINED,WINDOW_WIDTH,WINDOW_WIDTH,0);
    SDL_Renderer *renderer = SDL_CreateRenderer(window,-1,0);
    SDL_Texture  *texture  = SDL_CreateTexture(renderer,SDL_PIXELFORMAT_ARGB8888,SDL_TEXTUREACCESS_STATIC,WINDOW_WIDTH,WINDOW_WIDTH);
    SDL_SetRenderDrawColor(renderer,0,0,0,0);
    SDL_RenderClear(renderer);
    SDL_RenderPresent(renderer);

    cout<<prg_info<<"\n";
    cout<<"--------------------------------------\n";
    cout<<"Calculating a Mandelbrot of 600x600 pixels at 4096 iterations and looping 10 times\n";
    cout<<"Start...\n";

    uint64_t start = SDL_GetPerformanceCounter();
    uint64_t global_iterations = CalcMandelbrotASM_FracNEON_sp_opt1(pixels);
    uint64_t end   = SDL_GetPerformanceCounter();
    float secondsElapsed =(end-start)/(float)SDL_GetPerformanceFrequency();
    cout<<"Done...\n";

    SDL_UpdateTexture(texture,NULL,pixels,WINDOW_WIDTH*sizeof(uint32_t)); //NULL = entire texture
    SDL_RenderCopy(renderer,texture,NULL,NULL); //NULL = entire texture, NULL = entire destination
    SDL_RenderPresent(renderer);

    cout<<"Iterations.: "<<global_iterations<<"\n";
    cout<<"Time taken.: "<<fixed<<setprecision(3)<<secondsElapsed<<" s\n";
    cout<<"Speed......: "<<fixed<<setprecision(3)<<(global_iterations/secondsElapsed/1000000)<<" Million Iterations/s\n";
    cout<<"Close graphics window to quit...\n";

    while (1) {if (SDL_PollEvent(&event) && event.type == SDL_QUIT) break;}

    delete[] pixels;
    SDL_DestroyTexture(texture);
    SDL_DestroyRenderer(renderer);
    SDL_DestroyWindow(window);
    SDL_Quit();

    return 0;
}
